# Standard library imports:
import logging
import re


_LOG = logging.getLogger("p3.base")


# List of keywords taken from:
# http://www.python.org/doc/2.6/reference/lexical_analysis.html#id6
PYTHON_26_KEYWORDS = (
    "and",      "del",     "from",   "not",   "while",
    "as",       "elif",    "global", "or",    "with",
    "assert",   "else",    "if",     "pass",  "yield",
    "break",    "except",  "import", "print",
    "class",    "exec",    "in",     "raise",
    "continue", "finally", "is",     "return",
    "def",      "for",     "lambda", "try")


def is_all_caps_identifier_string(s):
    if not isinstance(s, basestring):
        return False
    else:
        # See comment regarding re.MULTILINE in is_identifier_string function.
        return re.match("^[A-Z_]{1}[A-Z0-9_]*$", s+'\n') is not None


def is_identifier_string(s):
    if not isinstance(s, basestring):
        return False
    else:
        # When re.MULTILINE option is not specified (this case), '$' pattern character matches at
        # the end of the string and immediately before the newline (if any) at the end of the
        # string.
        # To make this trailing newline significant for match result, additional trailing newline
        # is added to the string.
        return re.match("^[a-zA-Z_]{1}[a-zA-Z0-9_]*$", s+'\n') is not None


def is_internal_identifier_string(s):
    if not isinstance(s, basestring):
        return False
    else:
        return is_identifier_string(s) and re.match("^_+", s) is not None


def is_keyword_string(s):
    return s in PYTHON_26_KEYWORDS


def safe_fmt(fmt, *args):
    try:
        return fmt % args
    except TypeError:
        return fmt


def any_str(val):
    if isinstance(val, basestring):
        return uni_str(val)
    elif (isinstance(val, object) and hasattr(val, '__str__') and
          hasattr(val, '__unicode__') and (val.__class__.__str__ is object.__str__)):
        # Objects of classes derived from the 'object' always have '__str__' method
        # defined. If it's not not overridden in child classes and '__unicode__' method is provided
        # instead, unicode conversion will be used to convert the value to string. Such
        # implementation is used because default '__str__' method is not very useful.
        return uni_str(unicode(val))
    elif hasattr(val, '__str__'):
        return str(val)
    elif hasattr(val, '__unicode__'):
        return uni_str(unicode(val))
    else:
        return val.__class__.__name__


def str_uni(s):
    assert isinstance(s, basestring)

    if isinstance(s, str):
        return s.decode("utf8", "replace")
    else:
        return s



def uni_str(s):
    assert isinstance(s, basestring)

    if isinstance(s, unicode):
        return s.encode("utf8")
    else:
        return s
