# Standard library imports:
import itertools
import logging

# P3 imports:
from p3.base import Enum
from p3.data.error import DataModuleError
from p3.data.row_constructors import DefaultRowConstructor, OrderedRowConstructor
from p3.data.table import Table

JOIN_TYPE = Enum("inner", "left")

_LOG = logging.getLogger("p3.data")


def join_transform(table_1, table_2, index_cols, join_type=JOIN_TYPE.inner):
    if isinstance(index_cols, basestring):
        table_1_idx = tuple(c.index for c in table_1.resolve_columns(index_cols))
        table_2_idx = tuple(c.index for c in table_2.resolve_columns(index_cols))
    else:
        table_1_idx = tuple(c.index for c in table_1.resolve_columns(*index_cols))
        table_2_idx = tuple(c.index for c in table_2.resolve_columns(*index_cols))

    table_2_cols = tuple(c for c in table_2.cols if c.index not in table_2_idx)
    table_3 = Table.create_new(*itertools.chain(table_1.cols, table_2_cols))

    def make_key(row, col_idx):
        return tuple(row[i] for i in col_idx)

    table_2_lut = dict((make_key(r, table_2_idx), r) for r in table_2)

    for row_1 in table_1:
        key = make_key(row_1, table_1_idx)
        row_2 = table_2_lut.get(key)

        if row_2 is not None:
            table_3.add_row(
                OrderedRowConstructor(
                    tuple(c for c in row_1) +
                    tuple(c.value for c in row_2.cells if c.col in table_2_cols)))
        elif join_type == JOIN_TYPE.left:
            new_row = table_3.add_row(DefaultRowConstructor())
            for dst_cell, src_cell in zip(new_row._cells[:len(row_1)], row_1.cells):
                dst_cell.value = src_cell.value

    return table_3
